@@ -1,6 +1,7 @@
 #![allow(incomplete_features)]
 #![feature(generic_const_exprs)]
 #![feature(array_methods)]
+#![feature(array_chunks)]
 //! An extremely minimal super static type safe implementation of matrix types.
 //!
 //! While [`ndarray`](https://docs.rs/ndarray/latest/ndarray/) offers no compile time type checking
@@ -13,7 +14,7 @@
 //!  now both the number of rows and columns are known at compile time. This then allows this
 //!  infomation to propagate through your program providing excellent compile time checking.
 //!
-//! That being said... I made this in a weekend, there is a tiny amount of functionality and its ~4x slower than [`ndarray`](https://docs.rs/ndarray/latest/ndarray/) and [`nalgebra`](https://docs.rs/nalgebra/latest/nalgebra/).
+//! That being said... I made this in a weekend and there is a tiny amount of functionality.
 //!
 //! An example of how types will propagate through a program:
 //! ```ignore
@@ -88,42 +89,49 @@ pub use matmul::Matmul;
 #[derive(Eq, PartialEq, Debug, Clone)]
 pub struct MatrixDxD<T> {
     /// Underlying data.
-    pub data: Vec<Vec<T>>,
+    data: Vec<T>,
+    columns: usize,
+    rows: usize,
 }
 impl<T> MatrixDxD<T> {
     /// Number of elements in matrix.
     pub fn len(&self) -> usize {
-        self.data.len() * self.data[0].len()
-    }
-    /// Number of rows.
-    pub fn rows(&self) -> usize {
         self.data.len()
     }
     /// Number of columns.
     pub fn columns(&self) -> usize {
-        self.data[0].len()
+        self.columns
+    }
+    /// Number of rows.
+    pub fn rows(&self) -> usize {
+        self.rows
     }
 }
 impl<T: std::iter::Sum<T> + Clone> MatrixDxD<T> {
     /// Gets sum of all elements.
     pub fn sum(&self) -> T {
-        self.data.iter().map(|r| r.iter().cloned().sum()).sum()
+        self.data.iter().cloned().sum()
     }
     /// Gets the sum of each row.
     pub fn row_sum(&self) -> ColumnVectorD<T> {
         ColumnVectorD::from(
             self.data
-                .iter()
+                .chunks_exact(self.columns)
                 .map(|r| [r.iter().cloned().sum()])
                 .collect::<Vec<_>>(),
         )
     }
     /// Gets the sum of each column.
     pub fn column_sum(&self) -> RowVectorD<T> {
-        RowVectorD::try_from([self
-            .data
-            .iter()
-            .map(|r| r.iter().cloned().sum())
+        RowVectorD::try_from([(0..self.columns)
+            .map(|i| {
+                self.data
+                    .iter()
+                    .skip(i)
+                    .step_by(self.columns)
+                    .cloned()
+                    .sum()
+            })
             .collect::<Vec<_>>()])
         .unwrap()
     }
@@ -135,32 +143,37 @@ impl<T: std::iter::Sum<T> + Clone> MatrixDxD<T> {
 #[derive(Eq, PartialEq, Debug, Clone)]
 pub struct MatrixDxS<T, const COLUMNS: usize> {
     /// Underlying data.
-    pub data: Vec<[T; COLUMNS]>,
+    data: Vec<T>,
+    rows: usize,
 }
 impl<T, const COLUMNS: usize> MatrixDxS<T, COLUMNS> {
     /// Number of elements in matrix.
     pub fn len(&self) -> usize {
-        self.data.len() * COLUMNS
-    }
-    /// Number of rows.
-    pub fn rows(&self) -> usize {
         self.data.len()
     }
     /// Number of columns.
     pub const fn columns(&self) -> usize {
         COLUMNS
     }
+    /// Number of rows.
+    pub fn rows(&self) -> usize {
+        self.rows
+    }
 }
-impl<T: std::iter::Sum<T> + Copy + Default, const COLUMNS: usize> MatrixDxS<T, COLUMNS> {
+impl<T: std::iter::Sum<T> + Copy + Default + std::fmt::Debug, const COLUMNS: usize>
+    MatrixDxS<T, COLUMNS>
+where
+    [(); 1 * COLUMNS]:,
+{
     /// Gets sum of all elements.
     pub fn sum(&self) -> T {
-        self.data.iter().map(|r| r.iter().cloned().sum()).sum()
+        self.data.iter().cloned().sum()
     }
     /// Gets the sum of each row.
     pub fn row_sum(&self) -> ColumnVectorD<T> {
         ColumnVectorD::from(
             self.data
-                .iter()
+                .chunks_exact(COLUMNS)
                 .map(|r| [r.iter().cloned().sum()])
                 .collect::<Vec<_>>(),
         )
@@ -170,9 +183,8 @@ impl<T: std::iter::Sum<T> + Copy + Default, const COLUMNS: usize> MatrixDxS<T, C
         // Will dropping the `Default` and `Copy` traits to use `try_into` affect performance?
         let mut sums: [T; COLUMNS] = [Default::default(); COLUMNS];
         for i in 0..COLUMNS {
-            sums[i] = self.data.iter().map(|r| r[i]).sum();
+            sums[i] = self.data.iter().skip(i).step_by(COLUMNS).cloned().sum();
         }
-        // let vec = (0..COLUMNS).map(|c|self.data.iter().map(|r|r.nth(c)).sum()).collect::<Vec<_>>();
         RowVectorS::from([sums])
     }
 }
@@ -184,12 +196,13 @@ impl<T: std::iter::Sum<T> + Copy + Default, const COLUMNS: usize> MatrixDxS<T, C
 #[derive(Eq, PartialEq, Debug, Clone)]
 pub struct MatrixSxD<T, const ROWS: usize> {
     /// Underlying data.
-    pub data: [Vec<T>; ROWS],
+    data: Vec<T>,
+    columns: usize,
 }
 impl<T, const ROWS: usize> MatrixSxD<T, ROWS> {
     /// Number of elements in matrix.
     pub fn len(&self) -> usize {
-        ROWS * self.data[0].len()
+        self.data.len()
     }
     /// Number of rows.
     pub const fn rows(&self) -> usize {
@@ -197,29 +210,37 @@ impl<T, const ROWS: usize> MatrixSxD<T, ROWS> {
     }
     /// Number of columns.
     pub fn columns(&self) -> usize {
-        self.data[0].len()
+        self.columns
     }
 }
 use std::convert::TryFrom;
-impl<T: std::iter::Sum<T> + Copy + Default, const ROWS: usize> MatrixSxD<T, ROWS> {
+impl<T: std::iter::Sum<T> + Copy + Default + std::fmt::Debug, const ROWS: usize> MatrixSxD<T, ROWS>
+where
+    [(); ROWS * 1]:,
+{
     /// Gets sum of all elements.
     pub fn sum(&self) -> T {
-        self.data.iter().map(|r| r.iter().cloned().sum()).sum()
+        self.data.iter().cloned().sum()
     }
     /// Gets the sum of each row.
     pub fn row_sum(&self) -> ColumnVectorS<T, ROWS> {
         let mut sums: [[T; 1]; ROWS] = [[Default::default(); 1]; ROWS];
-        for i in 0..ROWS {
-            sums[i][0] = self.data[i].iter().cloned().sum();
+        for (i, row) in (0..ROWS).zip(self.data.chunks_exact(self.columns)) {
+            sums[i][0] = row.iter().cloned().sum();
         }
         ColumnVectorS::from(sums)
     }
     /// Gets the sum of each column.
     pub fn column_sum(&self) -> RowVectorD<T> {
-        RowVectorD::try_from([self
-            .data
-            .iter()
-            .map(|r| r.iter().cloned().sum())
+        RowVectorD::try_from([(0..self.columns)
+            .map(|i| {
+                self.data
+                    .iter()
+                    .skip(i)
+                    .step_by(self.columns)
+                    .cloned()
+                    .sum()
+            })
             .collect::<Vec<_>>()])
         .unwrap()
     }
@@ -229,11 +250,17 @@ impl<T: std::iter::Sum<T> + Copy + Default, const ROWS: usize> MatrixSxD<T, ROWS
 /// let _ = static_la::MatrixSxS::from([[1, 2, 3], [4, 5, 6]]);
 /// ```
 #[derive(Eq, PartialEq, Debug, Clone)]
-pub struct MatrixSxS<T, const ROWS: usize, const COLUMNS: usize> {
+pub struct MatrixSxS<T, const ROWS: usize, const COLUMNS: usize>
+where
+    [(); ROWS * COLUMNS]:,
+{
     /// Underlying data.
-    pub data: [[T; COLUMNS]; ROWS],
+    pub data: [T; ROWS * COLUMNS],
 }
-impl<T, const ROWS: usize, const COLUMNS: usize> MatrixSxS<T, ROWS, COLUMNS> {
+impl<T, const ROWS: usize, const COLUMNS: usize> MatrixSxS<T, ROWS, COLUMNS>
+where
+    [(); ROWS * COLUMNS]:,
+{
     /// Number of elements in matrix.
     pub const fn len(&self) -> usize {
         ROWS * COLUMNS
@@ -247,18 +274,25 @@ impl<T, const ROWS: usize, const COLUMNS: usize> MatrixSxS<T, ROWS, COLUMNS> {
         COLUMNS
     }
 }
-impl<T: std::iter::Sum<T> + Copy + Default, const ROWS: usize, const COLUMNS: usize>
-    MatrixSxS<T, ROWS, COLUMNS>
+impl<
+        T: std::iter::Sum<T> + Copy + Default + std::fmt::Debug,
+        const ROWS: usize,
+        const COLUMNS: usize,
+    > MatrixSxS<T, ROWS, COLUMNS>
+where
+    [(); ROWS * COLUMNS]:,
+    [(); ROWS * 1]:,
+    [(); 1 * COLUMNS]:,
 {
     /// Gets sum of all elements.
     pub fn sum(&self) -> T {
-        self.data.iter().map(|r| r.iter().cloned().sum()).sum()
+        self.data.iter().cloned().sum()
     }
     /// Gets the sum of each row.
     pub fn row_sum(&self) -> ColumnVectorS<T, ROWS> {
         let mut sums: [[T; 1]; ROWS] = [[Default::default(); 1]; ROWS];
-        for i in 0..ROWS {
-            sums[i][0] = self.data[i].iter().cloned().sum();
+        for (i, row) in (0..ROWS).zip(self.data.chunks_exact(COLUMNS)) {
+            sums[i][0] = row.iter().cloned().sum();
         }
         ColumnVectorS::from(sums)
     }
@@ -266,7 +300,7 @@ impl<T: std::iter::Sum<T> + Copy + Default, const ROWS: usize, const COLUMNS: us
     pub fn column_sum(&self) -> RowVectorS<T, COLUMNS> {
         let mut sums: [T; COLUMNS] = [Default::default(); COLUMNS];
         for i in 0..COLUMNS {
-            sums[i] = self.data.iter().map(|r| r[i]).sum();
+            sums[i] = self.data.iter().skip(i).step_by(COLUMNS).cloned().sum();
         }
         RowVectorS::from([sums])
     }
